
/////////////////////////////////////////////////////////////////////////////////
///////////////////////MODEL//////////////////////////////////////////////////////
var model = {
	message: "", //assignment or error message
	groups: [
		//group 0
		{
			input: {int4:-1, t2ck: -1, ic4:-1, ss1:-1, refclk1:-1}, //object used to hold the checked or not checked state of input peripherals in group 1
			inputPeripheral: ["int4", "t2ck", "ic4", "ss1", "refclk1"],
			inputChecked: [], //array to hold the checked peripherals
			output: {u1tx:-1, u2rts_n:-1, ss1_o:-1, oc1:-1, c2out:-1},
			outputPeripheral: ["u1tx", "u2rts_n", "ss1_o", "oc1", "c2out"],
			outputPeripheralValue: {u1tx:"0x0001", u2rts_n:"0x0002", ss1_o:"0x0003", oc1:"0x0005", c2out:"0x0007"}, //RPnR value to OUTPUT peripherals 
			outputChecked: [],
			pin: {pin2: 1, pin7: 1, pin11: 1, pin16: 1, pin26: 1},
			pinName: ["pin2", "pin7", "pin11", "pin16", "pin26"],
			pinReg: {pin2:"RPA0", pin7:"RPB3", pin11:"RPB4", pin16:"RPB7", pin26:"RPB15"}, //pin,register name dictionary
			pinValue: {pin2:"0x0000", pin7:"0x0001", pin11:"0x0002", pin16:"0x0004", pin26:"0x0003"}, //pin value for INPUT peripherals
			pinLeft: []	
		},
		//group 1
		{
			input: {int3: -1, t3ck: -1, ic3: -1, u1cts: -1, u2rx: -1, sdi1: -1},
			inputPeripheral: ["int3", "t3ck", "ic3", "u1cts", "u2rx", "sdi1"],
			inputChecked: [],
			output: {oc2: -1, c3out: -1},
			outputPeripheral: ["oc2", "c3out"],
			outputPeripheralValue: {oc2: "0x0005", c3out: "0x0007"},
			outputChecked: [],
			pin: {pin3: 1, pin5: 1, pin14: 1, pin17: 1, pin22: 1},
			pinName: ["pin3", "pin5", "pin14", "pin17", "pin22"],
			pinReg: {pin3: "RPA1", pin5: "RPB1", pin14: "PRB5", pin17: "RPB8", pin22: "RPB11"},
			pinValue: {pin3: "0x0000", pin5: "0x0002", pin14: "0x0001", pin17: "0x0004", pin22: "0x0003"},
			pinLeft: []
		},
		//group 2
		{
			input: {int2: -1, t4ck: -1, ic1: -1, ic5: -1, u1rx: -1, u2cts: -1, sdi2: -1, ocfb: -1},
			inputPeripheral: ["int2", "t4ck", "ic1", "ic5", "u1rx", "u2cts", "sdi2", "ocfb"],
			inputChecked: [],
			output: {oc4: -1, oc5: -1, refclko: -1},
			outputPeripheral: ["oc4", "oc5", "refclko"],
			outputPeripheralValue: {oc4: "0x0005", oc5: "0x0006", refclko: "0x0007"},
			outputChecked: [],
			pin: {pin6: 1, pin9: 1, pin12: 1, pin15: 1, pin24: 1},
			pinName: ["pin6", "pin9", "pin12", "pin15", "pin24"],
			pinReg: {pin6: "RPB2", pin9: "RPA2", pin12: "RPA4", pin15: "RPB6", pin24: "RPB13"},
			pinValue: {pin6: "0x0004", pin9: "0x0000", pin12: "0x0002", pin15: "0x0001", pin24: "0x0003"},
			pinLeft: []
		},
		//group 3
		{
			input: {int1: -1, t5ck: -1, ic2: -1, ss2: -1, ocfa: -1},
			inputPeripheral: ["int1", "t5ck", "ic2", "ss2", "ocfa"],
			inputChecked: [],
			output: {u1rts_n: -1, u2tx: -1, ss2_n: -1, oc3: -1, c1out: -1},
			outputPeripheral: ["u1rts_n", "u2tx", "ss2_n", "oc3", "c1out"],
			outputPeripheralValue: {u1rts_n: "0x0001", u2tx: "0x0002", ss2_n: "0x0004", oc3: "0x0005", c1out: "0x0007"},
			outputChecked: [],
			pin: {pin4: 1, pin10: 1, pin18: 1, pin21: 1, pin25: 1},
			pinName: ["pin4", "pin10", "pin18", "pin21", "pin25"],
			pinReg: {pin4: "RPB0", pin10: "RPA3", pin18: "RPB9", pin21: "RPB10", pin25: "RPB14"},
			pinValue: {pin4: "0x0002", pin10: "0x0000", pin18: "0x0004", pin21: "0x0003", pin25: "0x0001"},
			pinLeft: []
		},
		{
			output: {sdo1: -1, sdo2: -1},
			outputPeripheral: ["sdo1", "sdo2"],
			outputPeripheralValue: {sdo1: "0x0003", sdo2: "0x0004"},
			outputChecked: []
		}
	] //end of list groups[]
	
};


///////////////////////////////CONTROLLER/////////////////////////////////////////////////////////////
var controller = {
	
	init: function(){
		
		pinSelectionView.init();
		model.message = "";
	},
	
	flip: function(name, array){ //select or deselect a peripheral or a pin
		array[name] = - array[name];
	},
	
	checkAllPins: function(){
		var i, j, temp;
		for (i = 0; i <= 3; i++){
			temp = model.groups[i];
			for (j = 0; j < temp.pinName.length; j++){
				temp.pin[temp.pinName[j]] = -1;
			}
		}
	},
	
	getCheckedPeripheral: function(){ 
		var i, j, temp;
		for(i = 0; i <= 3; i++){
			temp = model.groups[i];
			
			temp.inputChecked.splice(0, temp.inputChecked.length); 
			for(j = 0; j < temp.inputPeripheral.length; j++){
				if(temp.input[temp.inputPeripheral[j]] == 1){
					temp.inputChecked.push(temp.inputPeripheral[j]);
				}
			}
			
			temp.outputChecked.splice(0, temp.outputChecked.length); 
			for(j = 0; j < temp.outputPeripheral.length; j++){
				if(temp.output[temp.outputPeripheral[j]] == 1){
					temp.outputChecked.push(temp.outputPeripheral[j]);
				}
			}
			
			temp.pinLeft.splice(0, temp.pinName.length); 
			for(j = 0; j < temp.pinName.length; j++){
				if(temp.pin[temp.pinName[j]] == 1){
					temp.pinLeft.push(temp.pinName[j]);
				}
			}
		}
		
		temp = model.groups[4];
		temp.outputChecked.splice(0, temp.outputChecked.length); 
		for(j = 0; j <= temp.outputPeripheral.length; j++){
			if(temp.output[temp.outputPeripheral[j]] == 1){
				temp.outputChecked.push(temp.outputPeripheral[j]);
			}
		}
	},//end of getChecked()
	
	checkForError: function(){ //return true if there is error
		var error = false;
		var i, temp;
		model.message = "";	
		for (i = 0; i <= 3; i++){
			temp = model.groups[i];
			if ( (temp.inputChecked.length + temp.outputChecked.length) > temp.pinLeft.length ) {
				error = true;
				model.message += "<br />" + "Fail to assign ";
				if (temp.inputChecked.length > 0){
					model.message += temp.inputChecked.length.toString().bold() + "   input peripherals: ".bold() + temp.inputChecked;
				}
				if (temp.outputChecked.length > 0){
					model.message += " " + temp.outputChecked.length.toString().bold() +"  output peripherals: ".bold() + temp.outputChecked;
				}
				model.message += " to " + temp.pinLeft.length.toString().bold() + " pins: ".bold()+ temp.pinLeft;
			}
		}
		
		//sdo1, sdo2
		if(model.groups[1].pinLeft.length + model.groups[2].pinLeft.length - (model.groups[1].inputChecked.length + model.groups[1].outputChecked.length + model.groups[2].inputChecked.length + model.groups[2].outputChecked.length) < model.groups[4].outputChecked.length ){
			model.message += "<br /> No enough pins left for " + model.groups[4].outputChecked + ", which can be mapped to " + model.groups[1].pinName + " and " + model.groups[2].pinName;  
			error = true;
		}
		if (error == true){
			model.message += "<br /> <br /> *If possible, think about using similar peripherals in other groups, go to " + "Reference".bold() + " page to find similar peripherals"
			
		}
		return error;
	}, //end of checkForError()
	
	doAssignment: function(){
		if(controller.checkForError()){return;}
		var i, j, checked, pin, length, temp;	
		for (i = 0; i <= 3; i++ ){
			temp = model.groups[i];
			length = temp.inputChecked.length;
			if (length > 0){

				for(j = 0; j < length; j++){
					model.message += "<br />";
					checked = temp.inputChecked.pop();
					pin = temp.pinLeft.pop();
					model.message += checked.toUpperCase() +"R" + " = " + temp.pinValue[pin] + "; //" + "Configure the input " + checked.toUpperCase() + " to pin " +  temp.pinReg[pin] + " (" + pin + ")";
				}
			}			
		} //end of outer for loop
		
		model.message += "<br /><br />";	
		for (i = 0; i <= 3; i++ ){
			temp = model.groups[i];
			length = temp.outputChecked.length;
			if (length > 0){

				for (j = 0; j < length; j++){
					model.message += "<br />";
					checked = temp.outputChecked.pop();
					pin = temp.pinLeft.pop();
					model.message += temp.pinReg[pin] + "R" + " = " + temp.outputPeripheralValue[checked] + "; //" + "Configure the output " + checked.toUpperCase() + " to pin " + temp.pinReg[pin]  + " (" + pin + ")";
				}
			}			
		}
		
		//do pin assignment for sdo1 and sdo2
		temp = model.groups[4];
		length = temp.outputChecked.length;
		while(length > 0 && model.groups[1].pinLeft.length > 0){
			model.message += "<br />";
			checked = temp.outputChecked.pop();
			pin = model.groups[1].pinLeft.pop();
			model.message += model.groups[1].pinReg[pin] + "R" + " = " + temp.outputPeripheralValue[checked] + "; //" + "Configure the output " + checked.toUpperCase() + " to pin " + model.groups[1].pinReg[pin] + " (" + pin + ")";
			length--;
		}
		
		while(length > 0 && model.groups[2].pinLeft.length > 0){
			model.message += "<br />";
			checked = temp.outputChecked.pop();
			pin = model.groups[2].pinLeft.pop();
			model.message += model.groups[2].pinReg[pin] + "R" + " = " + temp.outputPeripheralValue[checked] + "; //" + "Configure the output " + checked.toUpperCase() + " to pin " + model.groups[2].pinReg[pin] + " (" + pin + ")";
			length--;
		}		
		
	}//end of doAssignment()
	
};


///////////////////////////////VIEW///////////////////////////////////////////////////////////
var pinSelectionView = {
	
	init: function(){ //add event handlers to all items
	
		function sortUnorderedList(ul, sortDescending, nameOfForm) {
			if (typeof ul == "string") ul = document.getElementById(nameOfForm);
	
			var lis = $("#" + nameOfForm+ " label");
			var vals = [];
	
			for (var i = 0, l = lis.length; i < l; i++)
				vals.push(lis[i].innerHTML);
			vals.sort();
			if (sortDescending) vals.reverse();
			for (var i = 0, l = lis.length; i < l; i++)
				lis[i].innerHTML = vals[i];
		}
		
		function showView(viewName) {
			$('.view').hide();
			$('#' + viewName).show();
		}
			
		$(document).ready(function(){
			
			sortUnorderedList("list", false, "inputForm");
			sortUnorderedList("list", false, "outputForm");
			
			//multiple pages set up
			$('[data-launch-view]').click(function (e) {
				e.preventDefault();
				var viewName = $(this).attr('data-launch-view');
				showView(viewName);
			});	
			
			
			$('#nav li a').click(function() {
				  $('#nav li').removeClass();
				  $(this).parent().addClass('current');
			});
			
			
			
			var i, j, temp;
			for(i = 0; i <= 3; i++){
				//add event handler to input peripherals
				temp = model.groups[i];
				for(j = 0; j < temp.inputPeripheral.length; j++){
					$("#"+ temp.inputPeripheral[j]).click((function(name, array){
						return function(){
							controller.flip(name, array);
						};
					})(temp.inputPeripheral[j], temp.input));	
				}
				//add event handler to output peripherals
				for(j = 0; j < temp.outputPeripheral.length; j++){
					$("#"+ temp.outputPeripheral[j]).click((function(name, array){
						return function(){
							controller.flip(name, array);
						};
					})(temp.outputPeripheral[j], temp.output));	
				}
				//add event handler to pins
				for(j = 0; j < temp.pinName.length; j++){
					$("#"+ temp.pinName[j]).click((function(name, array){
						return function(){
							controller.flip(name, array);
						};
					})(temp.pinName[j], temp.pin));	
				}				
				
			}
			//add even handler for sdo1 and sdo2(the only 2 output peripherals that map to two groups of pins)
			temp = model.groups[4];
			for(j = 0; j < temp.outputPeripheral.length; j++){
					$("#"+ temp.outputPeripheral[j]).click((function(name, array){
						return function(){
							controller.flip(name, array);
						};
					})(temp.outputPeripheral[j], temp.output));	
			}
			
			$("#checkAllPins").click(function(){
				controller.checkAllPins();
				$("#pinForm input").attr("checked",true);
			});
			
			$("#submit").click(function(){
				controller.getCheckedPeripheral();
				controller.doAssignment();
				$("#message").html(model.message);
				showView("site_content2");
				$('#nav li').removeClass();
				$("#assignment").addClass('current');
			});
			
			$(":checkbox").attr("autocomplete", "off");
			$("#reset").click(function(){
				$("input[type='checkbox']").attr("checked",false);
				history.go(0);
			});

		});//end of ready()
	}//end of init()
	
};


controller.init();
